/* Terraform - (C) 1997-2000 Robert Gasch (r.gasch@chello.nl)
 *  - http://212.187.12.197/RNG/terraform/
 *  - http://terraform.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "PlaceObjectOptions.h"
#include "FileIO.h"
#include "GlobalTrace.h"
#include "GlobalSanityCheck.h"
#include "TFOptions.h"			// for sealevel
#include "terraform.h"


#if 0
// dummy table to trick xgettext into extracting the translatable strings
char *foobar[] = {
			N_("Plastic Model"),
			N_("Real Sky"),
			N_("Real Sky with Sunbeams (Fast)"),
			N_("Real Sky with Sunbeams (Slow)"),
 };
#endif



/* 
 * constructor: initialize all the class members
 */
PlaceObjectOptions::PlaceObjectOptions ()
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ PlaceObjectOptions\n");

	d_hfsx = d_hfsz = 1000;
	d_hfsy = 333;

	d_floor = 0.33;
	d_ceil = 0.66;
	d_density = 0.5;
	d_pvar = 0.3;
	d_osx = d_osy = d_osz = 1;
	d_osxvar = d_osyvar = d_oszvar = 0.33;
	d_seed = 0;

	p_objName = strdup ("tf_object");
}


/* 
 * destructor: delete any dynamically allocated data memebers
 */
PlaceObjectOptions::~PlaceObjectOptions ()
{
	if (p_objName)
		free (p_objName);

	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- PlaceObjectOptions\n");
}


/** 
 * writePOVFile: write out a POVRay-style file with the render settings
 */
int PlaceObjectOptions::writePOVFile (const char *fname) const
{
	SanityCheck::bailout ((!fname), "fname==NULL", "PlaceObjectOptions::writePOVFile");

	FileIO	*fPOV = new FileIO (fname);
	FILE	*fp;
	//float	ys = (b_halfYScale ? TFOptions::s_yscale/2 : TFOptions::s_yscale);

	fp = fPOV->open ("w");
	if (!fp)
		return -1;

	fPOV->close ();
	return 0;
}



/* ******************************************************************** */
/* ** mutators to set parameters and do some limited sanity checking ** */
/* ******************************************************************** */
void PlaceObjectOptions::setPlacementDensity (float density)
{
	d_density = density;
}


void PlaceObjectOptions::setPlacementVariance (float pvar)
{
	d_pvar = pvar;
}


void PlaceObjectOptions::setFloor (float floor)
{
	d_floor = floor;
}


void PlaceObjectOptions::setCeil (float ceil)
{
	d_ceil = ceil;
}


void PlaceObjectOptions::setHFScaling (int x, int y, int z)
{
	d_hfsx = x;
	d_hfsx = y; 
	d_hfsz = z; 
}


void PlaceObjectOptions::setObjScaling (float x, float y, float z)
{
	d_osx = x;
	d_osy = y; 
	d_osz = z; 
}


void PlaceObjectOptions::setObjScalingVariance (float x, float y, float z)
{
	d_osxvar = x;
	d_osyvar = y; 
	d_oszvar = z; 
}


void PlaceObjectOptions::setObjName (const char *objname)
{
	SanityCheck::bailout ((!objname), "objname==NULL", "PlaceObjectOptions::setObjName");

	if (p_objName)
		free (p_objName);
	p_objName = strdup (objname);
}


void PlaceObjectOptions::setSeed (int seed)
{
	d_seed = seed;
}


/* ******************************************************************** */
/* **************** accessors to return parameters ******************** */
/* ******************************************************************** */

float PlaceObjectOptions::getPlacementDensity () const
{
	return d_density;
}


float PlaceObjectOptions::getPlacementVariance () const
{
	return d_pvar;
}


float PlaceObjectOptions::getFloor () const
{
	return d_floor;
}


float PlaceObjectOptions::getCeil () const
{
	return d_ceil;
}


void PlaceObjectOptions::getHFScaling (int *x, int *y, int *z) const
{
	if (x)
		*x = d_hfsx;
	if (y)
		*y = d_hfsy;
	if (z)
		*z = d_hfsz;
}


void PlaceObjectOptions::getObjScaling (float *x, float *y, float *z) const
{
	if (x)
		*x = d_osx;
	if (y)
		*y = d_osy;
	if (z)
		*z = d_osz;
}


void PlaceObjectOptions::getObjScalingVariance (float *x, float *y, float *z) const
{
	if (x)
		*x = d_osxvar;
	if (y)
		*y = d_osyvar;
	if (z)
		*z = d_oszvar;
}


const char *PlaceObjectOptions::getObjName () const
{
	return p_objName;
}


int PlaceObjectOptions::getSeed () const
{
	return d_seed;
}

