/* Terraform - (C) 1997-2000 Robert Gasch (r.gasch@chello.nl)
 *  - http://212.187.12.197/RNG/terraform/
 *  - http://terraform.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#ifndef _PLACE_OBJECT_OPTIONS_H
#define _PLACE_OBJECT_OPTIONS_H 1


#include "GlobalDefs.h"


/**
 *  PlaceObjectOptions: options used exclusively for photorealistic rendering. 
 */
class PlaceObjectOptions 
	{
	public:
				PlaceObjectOptions ();
				~PlaceObjectOptions ();
		int		writePOVFile (const char *fname) const;

		// accessors
		void		getHFScaling (int *hfSx, int *hfSy, int *hfSz) const;
		void		getObjScaling (float *osx, float *osy, float *osz) const;
		void		getObjScalingVariance (float *osxv, float *osyv, float *vosz) const;
		float		getPlacementDensity() const;
		float		getPlacementVariance() const;
		float		getFloor () const;
		float		getCeil () const; 
		const char	*getObjName () const;
		int		getSeed () const;

		// mutators
		void		setHFScaling (int hfSx, int hfSy, int hfSz);
		void		setObjScaling (float osx, float osy, float osz);
		void		setObjScalingVariance (float osxv, float osyv, float vosz);
		void 		setPlacementDensity (float density);
		void 		setPlacementVariance (float variance);
		void 		setFloor (float floor);
		void 		setCeil (float ceil);
		void		setObjName (const char *objname);
		void		setSeed (int seed);

	private: 
		int		d_hfsx, d_hfsy, d_hfsz,	// HF scale factors
				d_seed;			// random # seed
		float		d_floor, d_ceil,	// target elevation
				d_density, d_pvar,	// density, placement variance
				d_osx, d_osy, d_osz,	// object scale factor
				d_osxvar, d_osyvar, d_oszvar;	// osf variance
		
		char 		*p_objName;	// the object name we place
	};


#endif // _PLACE_OBJECT_OPTIONS_H
