/* Terraform - (C) 1997-2000 Robert Gasch (r.gasch@chello.nl)
 *  - http://212.187.12.197/RNG/terraform/
 *  - http://terraform.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include <time.h>
#include <stdio.h>

#include "TFDialogGenPerlin.h"
#include "HeightFieldGenPerlin.h"
#include "MathRandom.h"
#include "TFGui.h"					// circular dependency
#include "TFOptions.h"
#include "TFWindowHandler.h"
#include "GlobalSanityCheck.h"
#include "GlobalTrace.h"
#include "agename.h"


#if 0
// dummy table to trick xgettext into extracting the translatable strings
char *foobar[] = {
			N_("Diamond Square"),
			N_("Midpoint Subdivision"),
			N_("Offset Square"),
			N_("Recursive Square Subdivision"),
			N_("Recursive Diamond Subdivision"),
};
#endif




/*
 *  constructor: initialize all data members 
 */
TFDialogGenPerlin::TFDialogGenPerlin ()
	        : GuiDialogOAC ("4 Layerd Perlin Noise Generator", NULL),
		  d_vbList1 (TRUE, 2),
		  d_vbList2 (TRUE, 2),
		  d_hbCategories (FALSE, 5),
		  d_hbParam1 (FALSE, 5),
		  d_hbParam2 (FALSE, 5),
		  d_hbParam3 (FALSE, 5),
		  d_hbParam4 (FALSE, 5),
		  d_hbSize (TRUE, 5),
		  d_hbGenSeed (TRUE, 5),
		  d_hbSeed (TRUE, 5), 
		  d_cb1 ("1st"), 
		  d_cb2 ("2nd"), 
		  d_cb3 ("3rd"), 
		  d_cb4 ("4th"),
		  d_frmPerlin ("Perlin Parameters"),
		  d_frmGen ("Generate Parameters")
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ TFDialogGenPerlin\n");

	p_HF=NULL;
	p_seedInitStr = new char [40];
	p_filters = NULL;
	reset ();

	d_mode = 0;

	p_cmbFilter1 = new  Gtk::Combo ();
	p_cmbFilter2 = new  Gtk::Combo ();
	p_cmbFilter3 = new  Gtk::Combo ();
	p_cmbFilter4 = new  Gtk::Combo ();
	p_lblFreq = new Gtk::Label (_("Frequency"));
	p_lblAmp = new Gtk::Label (_("Amplitude"));
	p_lblIter = new Gtk::Label (_("Iterations"));
	p_lblFilter = new Gtk::Label (_("Filter"));
	p_lblSize = new Gtk::Label (_("Size"));
	p_lblGenSeed = new Gtk::Label (_("Generate new seed"));
	p_lblSeed = new Gtk::Label (_("Specify Seed"));
	p_adjSize = new Gtk::Adjustment (400, 10, (TFOptions::s_large ? 5000 : 1000), 1);
	p_adjFreq1 = new Gtk::Adjustment (0.017, 0, (TFOptions::s_large ? 1 : 2), 0.001);
	p_adjFreq2 = new Gtk::Adjustment (0.021, 0, (TFOptions::s_large ? 1 : 2), 0.001);
	p_adjFreq3 = new Gtk::Adjustment (0.023, 0, (TFOptions::s_large ? 1 : 2), 0.001);
	p_adjFreq4 = new Gtk::Adjustment (0.025, 0, (TFOptions::s_large ? 1 : 2), 0.001);
	p_adjAmp1 = new Gtk::Adjustment (8192, 0, (TFOptions::s_large ? 1000 : 10000), 1);
	p_adjAmp2 = new Gtk::Adjustment (8192, 0, (TFOptions::s_large ? 1 : 2), 0.01);
	p_adjAmp3 = new Gtk::Adjustment (8192, 0, (TFOptions::s_large ? 1 : 2), 0.01);
	p_adjAmp4 = new Gtk::Adjustment (8192, 0, (TFOptions::s_large ? 1 : 2), 0.01);
	p_adjIter1 = new Gtk::Adjustment (5, 1, (TFOptions::s_large ? 100 : 1000), 1);
	p_adjIter2 = new Gtk::Adjustment (5, 1, (TFOptions::s_large ? 100 : 1000), 1);
	p_adjIter3 = new Gtk::Adjustment (5, 1, (TFOptions::s_large ? 100 : 1000), 1);
	p_adjIter4 = new Gtk::Adjustment (5, 1, (TFOptions::s_large ? 100 : 1000), 1);
	p_hsSize = new Gtk::HScale (*p_adjSize);
	p_spbFreq1 = new Gtk::SpinButton (*p_adjFreq1);
	p_spbFreq2 = new Gtk::SpinButton (*p_adjFreq2);
	p_spbFreq3 = new Gtk::SpinButton (*p_adjFreq3);
	p_spbFreq4 = new Gtk::SpinButton (*p_adjFreq4);
	p_spbAmp1 = new Gtk::SpinButton (*p_adjAmp1);
	p_spbAmp2 = new Gtk::SpinButton (*p_adjAmp2);
	p_spbAmp3 = new Gtk::SpinButton (*p_adjAmp3);
	p_spbAmp4 = new Gtk::SpinButton (*p_adjAmp4);
	p_spbIter1 = new Gtk::SpinButton (*p_adjIter1);
	p_spbIter2 = new Gtk::SpinButton (*p_adjIter2);
	p_spbIter3 = new Gtk::SpinButton (*p_adjIter3);
	p_spbIter4 = new Gtk::SpinButton (*p_adjIter4);

	buildDialogWindow ();
	handleCheckboxCallback (1);
	handleCheckboxCallback (2);
	handleCheckboxCallback (3);
	handleCheckboxCallback (4);
	d_cbGenSeed.clicked.connect(SigC::slot(this,&TFDialogGenPerlin::seedCheckboxCallback));
	//d_cb1.clicked.connect(SigC::slot(this,&TFDialogGenPerlin::seedCheckboxCallback), 1);
	//d_cb2.clicked.connect(SigC::slot(this,&TFDialogGenPerlin::seedCheckboxCallback), 2);
	//d_cb3.clicked.connect(SigC::slot(this,&TFDialogGenPerlin::seedCheckboxCallback), 3);
	d_cb1.clicked.connect(SigC::bind<int,void>(SigC::slot(this,&TFDialogGenPerlin::handleCheckboxCallback), 1));
	d_cb2.clicked.connect(SigC::bind<int,void>(SigC::slot(this,&TFDialogGenPerlin::handleCheckboxCallback), 2));
	d_cb3.clicked.connect(SigC::bind<int,void>(SigC::slot(this,&TFDialogGenPerlin::handleCheckboxCallback), 3));
	d_cb4.clicked.connect(SigC::bind<int,void>(SigC::slot(this,&TFDialogGenPerlin::handleCheckboxCallback), 4));

}



/*
 *  destructor: clean up, free allocated memory
 */
TFDialogGenPerlin::~TFDialogGenPerlin ()
{
	if (p_seedInitStr) delete p_seedInitStr;

	delete p_lblSize;
	delete p_lblGenSeed;
	delete p_lblSeed;
	delete p_adjSize;
	delete p_hsSize;

	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- TFDialogGenPerlin\n");
}


/*
 *  reset: clear p_tfWin so the next generate will register a new
 * 	window. Should be called before re-showing an existing dialog
 */
void TFDialogGenPerlin::reset ()
{
	p_HF = NULL;
	p_tfWin = NULL;
	this->set_title (_("4 Layerd Perlin Noise Generator"));
}


/*
 *  fillVBox: fill the dialog's VBox
 */
void TFDialogGenPerlin::fillVBox ()
{
	char	**s;

	// build Option Menu
	p_filters = s = new char*[MF];
	s[0] = _(FILTER_0);
	s[1] = _(FILTER_SQ1);
	s[2] = _(FILTER_SQ2);
	s[3] = _(FILTER_SQ3);
	s[4] = _(FILTER_ATAN1);
	s[5] = _(FILTER_ATAN2);
	s[6] = _(FILTER_ATAN3);
	s[7] = _(FILTER_ACOS1);
	s[8] = _(FILTER_ACOS2);
	s[9] = _(FILTER_ACOS3);
	s[10] = _(FILTER_COS1);
	s[11] = _(FILTER_COS2);
	s[12] = _(FILTER_EXP1);
	s[13] = _(FILTER_EXP2);
	s[14] = _(FILTER_EXP3);
	s[15] = _(FILTER_EXP4);
	s[16] = _(FILTER_EXP5);
	s[17] = _(FILTER_LOG1);
	s[18] = _(FILTER_LOG2);
	s[19] = _(FILTER_LOG3);
	s[20] = _(FILTER_TANH);
	s[21] = _(FILTER_SQRT);
	s[22] = NULL;

	Gtk::Table	*tbl = manage (new Gtk::Table(10, 5, FALSE));
	Gtk::HBox	*hb = manage (new Gtk::HBox(TRUE, 10));
	int		v=0;
	tbl->attach (*p_lblFreq, 2, 4, v, v+1);
	tbl->attach (*p_lblAmp, 4, 6, v, v+1);
	tbl->attach (*p_lblIter, 6, 8, v, v+1);
	tbl->attach (*p_lblFilter, 8, 10, v, v+1);
	//d_vbList1.pack_start (d_hbCategories, TRUE, TRUE, s_HBOff);

	v++;
	p_cmbFilter1->set_popdown_strings ((const char **)s);
	p_cmbFilter1->get_entry()->set_text (s[0]);
	p_cmbFilter1->get_entry()->set_editable (FALSE);
	tbl->attach (d_cb1, 0, 2, v, v+1);
	tbl->attach (*p_spbFreq1, 2, 4, v, v+1);
	tbl->attach (*p_spbAmp1, 4, 6, v, v+1);
	tbl->attach (*p_spbIter1, 6, 8, v, v+1);
	tbl->attach (*p_cmbFilter1, 8, 10, v, v+1);
	d_cb1.set_active (TRUE);
	p_spbFreq1->set_digits (3);
	p_spbAmp1->set_digits (0);
	p_spbIter1->set_digits (0);

	v++;
	p_cmbFilter2->set_popdown_strings ((const char **)s);
	p_cmbFilter2->get_entry()->set_text (s[0]);
	p_cmbFilter2->get_entry()->set_editable (FALSE);
	tbl->attach (d_cb2, 0, 2, v, v+1);
	tbl->attach (*p_spbFreq2, 2, 4, v, v+1);
	tbl->attach (*p_spbAmp2, 4, 6, v, v+1);
	tbl->attach (*p_spbIter2, 6, 8, v, v+1);
	tbl->attach (*p_cmbFilter2, 8, 10, v, v+1);
	p_spbFreq2->set_digits (3);
	p_spbAmp2->set_digits (0);
	p_spbIter2->set_digits (0);

	v++;
	p_cmbFilter3->set_popdown_strings ((const char **)s);
	p_cmbFilter3->get_entry()->set_text (s[0]);
	p_cmbFilter3->get_entry()->set_editable (FALSE);
	tbl->attach (d_cb3, 0, 2, v, v+1);
	tbl->attach (*p_spbFreq3, 2, 4, v, v+1);
	tbl->attach (*p_spbAmp3, 4, 6, v, v+1);
	tbl->attach (*p_spbIter3, 6, 8, v, v+1);
	tbl->attach (*p_cmbFilter3, 8, 10, v, v+1);
	p_spbFreq3->set_digits (3);
	p_spbAmp3->set_digits (0);
	p_spbIter3->set_digits (0);

	v++;
	p_cmbFilter4->set_popdown_strings ((const char **)s);
	p_cmbFilter4->get_entry()->set_text (s[0]);
	p_cmbFilter4->get_entry()->set_editable (FALSE);
	tbl->attach (d_cb4, 0, 2, v, v+1);
	tbl->attach (*p_spbFreq4, 2, 4, v, v+1);
	tbl->attach (*p_spbAmp4, 4, 6, v, v+1);
	tbl->attach (*p_spbIter4, 6, 8, v, v+1);
	tbl->attach (*p_cmbFilter4, 8, 10, v, v+1);
	p_spbFreq4->set_digits (3);
	p_spbAmp4->set_digits (0);
	p_spbIter4->set_digits (0);

	//d_vbList1.pack_start (*tbl, TRUE, TRUE, s_VBOff);
	hb->pack_start (*tbl, TRUE, TRUE, 20);
	d_vbList1.pack_start (*hb, TRUE, TRUE, s_VBOff);

	d_frmPerlin.set_border_width (5);
	d_frmPerlin.add (d_vbList1);

	//d_vbList.pack_start (d_sepMain, FALSE, FALSE, 0);
	//d_sepMain.show ();

	// pack size parameter
        d_hbSize.pack_start (*p_lblSize, TRUE, TRUE, s_HBOff);
        p_lblSize->show ();
        p_hsSize->set_update_policy (GTK_UPDATE_DELAYED);
	p_hsSize->set_adjustment (*p_adjSize);
        p_hsSize->set_digits (0);
        p_hsSize->set_draw_value (TRUE);
        d_hbSize.pack_end (*p_hsSize, TRUE, TRUE, s_HBOff);
        p_hsSize->show ();
	d_vbList2.pack_start (d_hbSize, TRUE, TRUE, 0);
	d_hbSize.show ();

	d_vbList2.pack_start (d_sepRnd, FALSE, FALSE, 0);
	d_sepRnd.show ();

	// random seed parameters 
	d_hbGenSeed.pack_start (*p_lblGenSeed, TRUE, TRUE, s_HBOff);
	p_lblGenSeed->show ();
	d_hbGenSeed.pack_end (d_cbGenSeed, TRUE, TRUE, s_HBOff);
	d_cbGenSeed.set_active (TRUE);
	d_cbGenSeed.show ();
	d_vbList2.pack_start (d_hbGenSeed, TRUE, TRUE, 0);
	d_hbGenSeed.show ();

	p_lblSeed->set_justify (GTK_JUSTIFY_RIGHT);
	d_hbSeed.pack_start (*p_lblSeed, TRUE, TRUE, s_HBOff);
	p_lblSeed->set_sensitive (FALSE);
	p_lblSeed->show ();

	MathRandom	*mRand = new MathRandom ();
	sprintf (p_seedInitStr, "%d", (int)mRand->rnd());
	delete mRand;
	d_enSeed.set_text (p_seedInitStr);
	d_enSeed.set_sensitive (FALSE);
	d_hbSeed.pack_end (d_enSeed, FALSE, FALSE, s_HBOff);
	d_enSeed.show ();
	d_vbList2.pack_start (d_hbSeed, TRUE, TRUE, 0);
	d_hbSeed.show ();

	d_frmGen.set_border_width (5);
	d_frmGen.add (d_vbList2);

	this->get_vbox()->pack_start (d_frmPerlin);
	this->get_vbox()->pack_start (d_frmGen);
	this->show_all ();
	
}


/*
 *  buildDialogWindow:
 */
void TFDialogGenPerlin::buildDialogWindow ()
{
	this->set_usize (600, 400);
	this->get_vbox()->set_border_width (2);

	this->setupVBox ();
	fillVBox ();
	fillActionArea ();

	this->show ();
}



void TFDialogGenPerlin::buttonCallbackApply ()
{
	bool			b1, b2, b3, b4;
	float			f1, f2, f3, f4,
				a1, a2, a3, a4;
	int			i1, i2, i3, i4,
				e1, e2, e3, e4,
				i;
	//char			*s;
	string			s;
	int			size, seed;
	float			scaleFactor;
	string			entry;
	bool			HFisNew = FALSE;
	HeightFieldGenPerlin	*HFGen;

	if (!p_HF)
		{
		p_HF = new HeightField ();
		HFisNew = TRUE;
		}
	HFGen = new HeightFieldGenPerlin (p_HF);

	// get slider values
	size = (int)(p_adjSize->get_value());
	b1 = d_cb1.get_active ();
	b2 = d_cb2.get_active ();
	b3 = d_cb3.get_active ();
	b4 = d_cb4.get_active ();
	f1 = p_spbFreq1->get_value_as_float ();
	f2 = p_spbFreq2->get_value_as_float ();
	f3 = p_spbFreq3->get_value_as_float ();
	f4 = p_spbFreq4->get_value_as_float ();
	a1 = p_spbAmp1->get_value_as_float ();
	a2 = p_spbAmp2->get_value_as_float ();
	a3 = p_spbAmp3->get_value_as_float ();
	a4 = p_spbAmp4->get_value_as_float ();
	i1 = p_spbIter1->get_value_as_int ();
	i2 = p_spbIter2->get_value_as_int ();
	i3 = p_spbIter3->get_value_as_int ();
	i4 = p_spbIter4->get_value_as_int ();

	s = (p_cmbFilter1->get_entry())->get_text();
	for (i=0; i<MF; i++)
		if (s == p_filters[i])
			{
			e1=i;
			i=MF;
			}
	
	s = (p_cmbFilter2->get_entry())->get_text();
	for (i=0; i<MF; i++)
		if (s == p_filters[i])
			{
			e2=i;
			i=MF;
			}
	
	s = (p_cmbFilter3->get_entry())->get_text();
	for (i=0; i<MF; i++)
		if (s == p_filters[i])
			{
			e3=i;
			i=MF;
			}

	s = (p_cmbFilter4->get_entry())->get_text();
	for (i=0; i<MF; i++)
		if (s == p_filters[i])
			{
			e4=i;
			i=MF;
			}

	// either generate or read a random number seed
	if (d_cbGenSeed.get_active())
		{
		MathRandom	*mRand = new MathRandom ();

		seed = (int) (mRand->rnd()*G_MAXINT);
		sprintf (p_seedInitStr, "%d", seed);
		d_enSeed.set_text (p_seedInitStr);
		delete mRand;
		}
	else
		{
		s = const_cast<char*>(d_enSeed.get_text().c_str());
		seed = atoi (s.c_str());
		}

	SanityCheck::bailout ((!HFGen), "HFGen==NULL", "TFDialogGenPerlin::buttonCallbackApply");

	HFGen->generate (size, size, seed, b1, b2, b3, b4,
			f1, f2, f3, f4, a1, a2, a3, a4, 
			i1+1, i2+2, i3+3, i4+4, e1, e2, e3, e4);
	delete HFGen;
	HFGen = NULL;

	if (HFisNew)
		{
		p_tfWin = TFGui::addHeightField (p_HF);
		this->set_title (p_HF->getName());
		}
	else
		p_tfWin->drawHeightField ();

	this->b_applyHit = TRUE;
}


void TFDialogGenPerlin::handleCheckboxCallback (int i)
{
        bool            status=d_enSeed.is_sensitive ();
	Gtk::SpinButton	*spbFreq, *spbAmp, *spbIter;
	Gtk::Combo	*cmbFilter;

	if (i == 1) 	
		{
		status = d_cb1.get_active ();
		spbFreq = p_spbFreq1; 
		spbAmp = p_spbAmp1; 
		spbIter = p_spbIter1;
		cmbFilter = p_cmbFilter1;
		}
	else 
	if (i == 2) 
		{
		status = d_cb2.get_active ();
		spbFreq = p_spbFreq2; 
		spbAmp = p_spbAmp2; 
		spbIter = p_spbIter2;
		cmbFilter = p_cmbFilter2;
		}
	else 
	if (i == 3) 
		{
		status = d_cb3.get_active ();
		spbFreq = p_spbFreq3; 
		spbAmp = p_spbAmp3; 
		spbIter = p_spbIter3;
		cmbFilter = p_cmbFilter3;
		}
	else 
	if (i == 4) 
		{
		status = d_cb4.get_active ();
		spbFreq = p_spbFreq4; 
		spbAmp = p_spbAmp4; 
		spbIter = p_spbIter4;
		cmbFilter = p_cmbFilter4;
		}

	spbFreq->set_sensitive (status); 
	spbAmp->set_sensitive (status); 
	spbIter->set_sensitive (status); 
	cmbFilter->set_sensitive (status); 
}


void TFDialogGenPerlin::seedCheckboxCallback ()
{
        //bool            status=d_enSeed.is_sensitive ();

        //p_lblSeed->set_sensitive (!status);
        //d_enSeed.set_sensitive (!status);
}


gint TFDialogGenPerlin::delete_event_impl (GdkEventAny *e)
{
	this->hide ();
	return (1);
}


